import 'dart:convert';
import 'dart:typed_data';

import 'package:flx_id_binder/flx_id_binder.dart';
import 'package:get/get.dart';

class AchievementBinder extends ApiBinderBase {
  final _storage = Get.find<ApiStorage>();

  Future<AchievementInfoModel> getInfo({
    required String applicationId,
  }) async {
    return await errorHandler.handle(() async {
      var result = await requestHandler.handleGet(
        "achievement",
        "info",
        apiVersion: 1,
        queries: {
          "applicationId": applicationId,
        },
      );

      return AchievementInfoModel.fromJson(result!);
    });
  }

  Future<void> create({
    required String applicationId,
    required String name,
    required String description,
    required Uint8List backgroundPicture,
    int score = 10,
    bool isRare = false,
  }) async {
    await errorHandler.handle(() async {
      await requestHandler.handlePost(
        "achievement",
        "create",
        apiVersion: 1,
        body: {
          "achievementName": name,
          "description": description,
          "backgroundPictureBase64": base64.encode(backgroundPicture),
        },
      );
    });
  }

  Future<void> permit({
    required String achievementId,
  }) async {
    await errorHandler.handle(() async {
      await requestHandler.handlePost(
        "achievement",
        "permit",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
        body: {
          "achievementId": achievementId,
        },
      );
    });
  }
}
