import 'dart:convert';
import 'dart:typed_data';

import 'package:flx_id_binder/flx_id_binder.dart';
import 'package:get/get.dart';

class ApplicationBinder extends ApiBinderBase {
  final _storage = Get.find<ApiStorage>();

  Future<ApplicationInfoModel> getInfo({
    required String applicationId,
  }) async {
    return await errorHandler.handle(() async {
      var result = await requestHandler.handleGet(
        "application",
        "info",
        apiVersion: 1,
        queries: {
          "applicationId": applicationId,
        },
      );

      return ApplicationInfoModel.fromJson(result!);
    });
  }

  Future<void> create({
    required String name,
    required Uint8List logoPicture,
  }) async {
    await errorHandler.handle(() async {
      await requestHandler.handleGet(
        "application",
        "create",
        apiVersion: 1,
        queries: {
          "name": name,
          "logoPictureBase64": base64.encode(logoPicture),
        },
      );
    });
  }

  Future<String> getSecret({
    required String applicationId,
  }) async {
    return await errorHandler.handle(() async {
      var result = await requestHandler.handleGet(
        "application",
        "secret",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
        queries: {
          "applicationId": applicationId,
        },
      );

      return result!["secret"];
    });
  }

  Future<List<String>> getAchievementsIds({
    required String applicationId,
  }) async {
    return await errorHandler.handle(() async {
      var result = await requestHandler.handleGet(
        "application",
        "achievements",
        apiVersion: 1,
        queries: {
          "applicationId": applicationId,
        },
      );

      return (result!["achievementsIds"] as List<dynamic>)
          .map((e) => e as String)
          .toList();
    });
  }
}
