import 'dart:convert';
import 'dart:typed_data';

import 'package:flx_id_binder/flx_id_binder.dart';
import 'package:get/get.dart';

class UserBinder extends ApiBinderBase {
  final _storage = Get.find<ApiStorage>();

  Future<UserInfoModel> getInfo() async {
    return await errorHandler.handle(() async {
      var result = await requestHandler.handleGet(
        "user",
        "info",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
      );

      return UserInfoModel.fromJson(result!);
    });
  }

  Future<void> setInfo({
    String? firstName,
    String? lastName,
    String? nickname,
  }) async {
    await errorHandler.handle(() async {
      await requestHandler.handlePost(
        "user",
        "info",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
        body: {
          "firstName": firstName,
          "lastName": lastName,
          "nickname": nickname,
        },
      );
    });
  }

  Future<Uint8List?> getPicture() async {
    return await errorHandler.handle(() async {
      var result = await requestHandler.handleGet(
        "user",
        "picture",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
      );

      var imageBase64 = result?["pictureBase64"] as String?;

      return imageBase64 != null ? base64.decode(imageBase64) : null;
    });
  }

  Future<void> setPicture({
    required Uint8List? imageBytes,
  }) async {
    await errorHandler.handle(() async {
      await requestHandler.handlePost(
        "user",
        "picture",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
        body: {
          "imageBase64": base64.encode(imageBytes!),
        },
      );
    });
  }

  Future<void> deletePicture() async {
    await errorHandler.handle(() async {
      await requestHandler.handleDelete(
        "user",
        "picture",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
      );
    });
  }

  Future<List<String>> getAppsIds() async {
    return await errorHandler.handle(() async {
      var result = await requestHandler.handleGet(
        "user",
        "applications",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
      );

      return (result!["applicationsIds"] as List<dynamic>)
          .map((e) => e as String)
          .toList();
    });
  }

  Future<List<String>> getAchievementsIds() async {
    return await errorHandler.handle(() async {
      var result = await requestHandler.handleGet(
        "user",
        "achievements",
        apiVersion: 1,
        headers: {
          "Authorization": "Bearer ${_storage.accessToken!}",
        },
      );

      return (result!["achievementsIds"] as List<dynamic>)
          .map((e) => e as String)
          .toList();
    });
  }
}
