import 'package:dio/dio.dart';
import 'package:flx_id_binder/flx_id_binder.dart';
import 'package:get/get.dart';

class ApiExceptionHandler extends GetxService {
  final handleExceptions = new Map<String, ApiException>();

  Future handle(Function function) async {
    try {
      return await function();
    } on DioError catch (e) {
      try {
        _handleDioError(e);
      } catch (e) {
        if (e is ApiException) {
          throw e;
        } else {
          throw UnhandleApiException();
        }
      }
    }
    throw UnhandleApiException();
  }

  void _handleDioError(DioError dioError) {
    var response = dioError.response;

    if (response == null || response.statusCode! >= 500) {
      throw new BackendApiException();
    } else {
      var data = response.data;
      var name = data["name"];
      var message = data["message"];

      var exception = handleExceptions[name];

      if (exception != null) {
        exception.message = message;
        throw exception;
      } else {
        var clientException = new ClientApiException();
        clientException.message = message;
        throw clientException;
      }
    }
  }
}
